; *****************************************************************************
; *** Soubor:	Macmod.inc													***
; ***																		***
; *** PLC pro plc stroje, definice maker (uivatelskch instrukc).		***
; ***																		***
; *** (C) MEFI s.r.o.														***
; *****************************************************************************

CONTROL_T_MACRO pos

; Makro SetMechPtr napln promnnou typu DS 2 ukazatelem na mechanizmus. Parametry:
; 	Ptr		- promnn, kter se m naplnit
;	Mech 	- mechanizmus
SetMechPtr macro Ptr,Mech
		mov		Ptr, offset DGROUP:Mech
Endm

; Makro LDR_MECH slou k naten stavu mechanizmu zadanho ukazatelem do RLO. Ukazatel
; na mechanizmus se pedv v parametru MechPtr. Pokud je ukazatel nulov, nate do 
; RLO 0, jinak stav mechanizmu, na kter ukazuje MechPtr.
DEF_T_MACRO	LDR_MECH MechPtr
T_LOCAL	LBL_LDR_MECH_NULL, LBL_LDR_MECH_NOTNULL, LBL_LDR_MECH_END
	PUSH	ECX						; Uloit DR na stack
	MOV		CX, MechPtr
	EQ		CNST.0
	JL1		LBL_LDR_MECH_NULL
LBL_LDR_MECH_NOTNULL:		
	MOV		DI, MechPtr				; Ukazatel je nenulov - nast ho
	LDR		[DI].b0					; 	a pest bit b0 z adresy, kam ukazuje
	JL0		LBL_LDR_MECH_END
	JL1		LBL_LDR_MECH_END
LBL_LDR_MECH_NULL:
	LDR		0
LBL_LDR_MECH_END:
	POP		ECX						; Obnovit DR ze stacku
END_T_MACRO T_PUSH, C_0, D_ON

; Makro RUN_MECH1 slou k podmnnmu sputn mechanizmu zadanho ukazatelem. Ukazatel na 
; mechanizmus se pedv v parametru MechPtr. Pokud je RLO=0 nebo ukazatel nulov, nedl 
; nic, jinak mechanizmus, na kter ukazuje MechPtr, spust.
DEF_T_MACRO	RUN_MECH1 MechPtr
T_LOCAL LBL_RUN_MECH1_END
		JL0		LBL_RUN_MECH1_END	; Pokud RLO=0, nic nedlat
		
		MOV		DI, MechPtr
		OR		DI, DI
		JZ		LBL_RUN_MECH1_END	; Pokud je ukazatel nulov, nic nedlat
		
		MOV		DI, MechPtr
		FL		1, [DI].b0			; Nastavit bit b0 na adrese, kam ukazuje ukazatel
LBL_RUN_MECH1_END:
END_T_MACRO T_END, C_0, D_ON

; Makro RUN_MECH slou ke sputn mechanizmu zadanho ukazatelem. Ukazatel na mechanizmus
; se pedv v parametru MechPtr. Pokud je ukazatel nulov, nedl nic, jinak mechanizmus,
; na kter ukazuje MechPtr, spust.
DEF_T_MACRO	RUN_MECH MechPtr
T_LOCAL LBL_RUN_MECH_END
		MOV		DI, MechPtr
		OR		DI, DI
		JZ		LBL_RUN_MECH_END	; Pokud je ukazatel nulov, nic nedlat

		MOV		DI, MechPtr
		FL		1, [DI].b0			; Nastavit bit b0 na adrese, kam ukazuje ukazatel
LBL_RUN_MECH_END:
END_T_MACRO T_NORMAL, C_0, D_ON

; Makro CALL1 provd podmnn voln procedury. Pokud je RLO=1 zavol proceduru Proc.
DEF_T_MACRO	CALL1 Proc
T_LOCAL LBL_CALL1_END
		; Pkud je RLO=0, nedlat nic (skoit na konec)
	JL0		LBL_CALL1_END
		; Uloit na zsobnk DR a RLO (aby byly k dispozici i po voln procedury)
	PUSH		ECX
	PUSH		AX
		; Zavolat proceduru
	CALL 	DWORD PTR POINT_PROC_&Proc
		; Obnovit ze zsobnku DR a RLO
	POP			AX
	POP			ECX
LBL_CALL1_END:
END_T_MACRO T_END, C_0, D_ON

; Makro TIMER slou jako nhrada instrukce TIM. Na rozdl od n umouje testovn podmnek
; pro pedasn ukonen asovn, ani by se TIM poutl ze samostatnho mechanizmu
; (makro je uritou obdobou instrukce TM). Ped volnm makra je poteba do RLO nastavit 1,
; pokud je poadovno pedasn ukonen asovn, jinak 0. Instrukce nastav RLO na 1, pokud
; bylo asovn dokoneno, jinak na 0.
; Parametry:
;	TimerVar		Promnn asovae, pi kadm prchodu se inkrementuje
;	Time			Doba asovn (jako poet takt)
DEF_T_MACRO	TIMER TimerVar, Time
T_LOCAL LBL_TIMER_END
	JL1		LBL_TIMER_END
		; Otestovat, zda u bylo dosaeno poadovanho asu. Pokud ano, skoit na konec (v RLO zstane 1)
	LOD		TMAC.TimerVar
	GE		TMAC.Time
	JL1		LBL_TIMER_END
		; Inkrementovat ta
	LOD		TMAC.TimerVar
	INR
	STO		TMAC.TimerVar
LBL_TIMER_END:
END_T_MACRO T_NORMAL, C_0, D_ON

; Makro SELECT_TORCH slou k usnadnn volby aktivnch hok. Pro hoky zadan maskou
; v parametru TorchMask zamn stavy aktivn/neaktivn (pouze pro hoky, pro kter je
; nastaven pslun bit v parametru UsedTorches). Uvedenou zmnu provd pouze tehdy, 
; pokud je RLO roven 1. Zachovv obsah DR i RLO.
; Parametry:
;	TorchMask		- Maska hok, pro kter m bt volba provedena.
;	ActiveTorches	- Promnn typu byte, jej jednotliv bity kaj, kter hoky jsou aktivn.
;	UsedTorches		- Promnn typu byte, jej jednotliv bity kaj, na kterch suportech je osazen hok dan technologie.
DEF_T_MACRO	SELECT_TORCH	TorchMask, ActiveTorches, UsedTorches
T_LOCAL LBL_SELECT_TORCH_END

	JL0		LBL_SELECT_TORCH_END	; Pokud je DR roven 0, skoit na konec makra (usnadnn pi pouit v udlostn procedue _ON_CMD)

	PUSH	EAX						; Uloit RLO na stack
	PUSH	ECX						; Uloit DR na stack
	LOD		TMAC.TorchMask
	ANDB	TMAC.UsedTorches
	XORB	TMAC.ActiveTorches
	STO		TMAC.ActiveTorches		; Inverze vech bit danch maskou TorchMask, kter maj nastaven bit v ActiveTorches
	POP		ECX						; Obnovit DR ze stacku
	POP		EAX						; Obnovit RLO ze stacku
	
LBL_SELECT_TORCH_END:
END_T_MACRO T_END, C_0, D_ON

; Makro LDR_TACTIVE slou k naten stavu jednoho suportu do RLO - pokud je suport aktivn 
; (alespo pro jednu technologii je stav nastaven na SEL_SELECTED), nate se do RLO 1, jinak 0.
DEF_T_MACRO	LDR_TACTIVE TNo
	LDR
	XOR		ECX, ECX
	MOV		CL, bT&TNo&SelOxy			; LOD			bT&TNo&SelOxy
	EQ		SEL_SELECTED
	LDR
	XOR		ECX, ECX
	MOV		CL, bT&TNo&SelPlasma		;	LOD		bT&TNo&SelPlasma
	EQ		SEL_SELECTED
	LO
	LDR
	XOR		ECX, ECX
	MOV		CL, bT&TNo&SelMarker		;	LOD		bT&TNo&SelMarker
	EQ		SEL_SELECTED
	LO
	LDR
	XOR		ECX, ECX
	MOV		CL, bT&TNo&SelWaterJet		;	LOD		bT&TNo&SelWaterJet
	EQ		SEL_SELECTED
	LO
END_T_MACRO T_PUSH, C_0, D_ON

; Makro SET_BIT usnaduje nastaven bitu v promnn. Pokud je RLO roven 1, nastav
; bit (bity) uren maskou v parametru Mask v promnn dan parametrem Var.
DEF_T_MACRO SET_BIT	Mask, Var
T_LOCAL LBL_SET_BIT_END

	JL0		LBL_SET_BIT_END		; Pokud je v RLO 0, nedlat nic
	
	LOD		TMAC.Var			; Nast promnnou
	ORB		TMAC.Mask			; Nastavit vechny byty, kter jsou v masce jednikov
	STO		TMAC.Var			; Uloit do promnn
	
LBL_SET_BIT_END:
END_T_MACRO T_END, C_0, D_ON

; Makro ORB1 udnaduje podmnn vykonn instrukce ORB: pokud je RLO roven 1,
; provede instrukci ORB, jinak nic.
DEF_T_MACRO ORB1 par
T_LOCAL LBL_ORB1_END

	JL0		LBL_ORB1_END
	ORB		TMAC.par
	
LBL_ORB1_END:
END_T_MACRO T_NORMAL, C_0, D_ON

;Definice uivatelsk instrukce LOD1, kter provd
;podmnn ten buky do DR v ppad, e RLO je 1.
DEF_T_MACRO LOD1 p1
T_LOCAL	END_LOD1

	JL0		END_LOD1
	LOD		TMAC.p1
END_LOD1:

END_T_MACRO T_NORMAL, C_0, D_ON

;.......

;Definice uivatelsk instrukce BIT_SEQ, kter pevede 1. nenulov bit na poadov slo do DR 
DEF_T_MACRO BIT_SEQ p1
T_LOCAL LBL_PREV2_EE,LBL_PREV2_CYK
			LOD		TMAC.p1
			mov		edx,ecx
			mov		ecx,1
		LBL_PREV2_CYK:
			shr		edx,1
			jb		LBL_PREV2_EE
			inc		cx
			or		edx,edx
			jnz		LBL_PREV2_CYK
			xor		ecx,ecx												;neni nastaven zaden bit
		LBL_PREV2_EE:
END_T_MACRO T_NORMAL, C_0, D_ON


;........
; Pevod asu z relnho  v sekundch na poet takt PLC
DEF_T_MACRO CONV_S_PLCCYCLE	Time, CycleCnt
		LOD		CNST.50
		REAL	DWRD
		MULB	REAL.Time
		STO		TMAC.CycleCnt
END_T_MACRO t_normal, c_0, d_on

;Prevod casu [s] z REAL do DWORD v taktu [20ms]
DEF_T_MACRO REAL_TIME20MS B_REAL,B_DWORD
		cli
		finit
		fld		qword ptr B_REAL
		fimul	word ptr BW50
		fistp	dword ptr B_DWORD
		sti  
END_T_MACRO t_normal, c_0, d_on

;........

;Prevod casu [s] z REAL do DWORD v promile
DEF_T_MACRO REAL_PROMILE B_REAL,B_DWORD
		cli
		finit
		fld		qword ptr B_REAL
		fimul	word ptr BW10
		fistp	dword ptr B_DWORD
		sti  
END_T_MACRO t_normal, c_0, d_on


;........

;Prevod casu [s] z REAL do DWORD v procentech
DEF_T_MACRO REAL_PROCENTO B_REAL,B_DWORD
		cli
		finit
		fld		qword ptr B_REAL
		fistp	dword ptr B_DWORD
		sti  
END_T_MACRO t_normal, c_0, d_on


;........
;Prevod casu [s] z REAL do DWORD v taktu [0.1ms]
DEF_T_MACRO REAL_TIME100MS B_REAL,B_DWORD
		cli
		finit
		fld		qword ptr B_REAL
		fimul	word ptr BW10
		fistp	dword ptr B_DWORD
		sti  
END_T_MACRO t_normal, c_0, d_on

;........

;Prevod casu [0.1s] z DWORD do REAL v taktu [s]
DEF_T_MACRO DWORD_TIME1S B_DWORD,B_REAL
		cli
		finit
		fild	dword ptr B_DWORD
		fidiv	word ptr BW10
		fstp	qword ptr B_REAL
		sti  
END_T_MACRO t_normal, c_0, d_on

;.........

;Prevod casu [20ms] z WORD do REAL v taktu [s]
DEF_T_MACRO DWORD_TIME02S B_WORD,B_REAL
		cli
		finit
		fild	word ptr B_WORD
		fidiv	word ptr BW50
		fstp	qword ptr B_REAL
		sti  
END_T_MACRO t_normal, c_0, d_on

;........

;Prevod z REAL do DWORD primo
DEF_T_MACRO REAL_DWORD B_REAL,B_DWORD
		cli
		finit
		fld		qword ptr B_REAL
		fistp	dword ptr B_DWORD
		sti  
END_T_MACRO t_normal, c_0, d_on

;........

;Prevod z DWORD do REAL primo
DEF_T_MACRO DWORD_REAL B_DWORD,B_REAL
		cli
		finit
		fild	dword ptr B_DWORD
		fstp	qword ptr B_REAL
		sti  
END_T_MACRO t_normal, c_0, d_on


;........

;Prevod miry [mm] z REAL do DWORD na [um]
DEF_T_MACRO REAL_MM B_REAL,B_WORD
		cli
		finit
		fld		qword ptr B_REAL
		fimul	word ptr BW1000
		fistp	dword ptr B_WORD
		sti  
END_T_MACRO t_normal, c_0, d_on

;........

;Prevod rychlosti [mm/min] z REAL do DWORD na [1/64000 mm/min (POS)]
DEF_T_MACRO REAL_MM_MIN_64 B_REAL,B_WORD
		cli
		finit
		fld		qword ptr B_REAL
		fistp	word ptr B_WORD+2
		sti  
END_T_MACRO t_normal, c_0, d_on

;........

;Prevod WORD na DWORD se znamenkovym rozsirenim
DEF_T_MACRO DWORD_SIG
			MOV		AX,CX				;!! ASM znamenkove rozsireni
			CWDE						;!! prevod na DWORD
			MOV		ECX,EAX				;!!
END_T_MACRO t_normal, c_0, d_on



; Makro PLC_MEM_READ slou k usnadnn ten ze zlohovan pamti PLC (PLC_MEM_BACKUP).
; Nate ze zadanho msta zlohovan pamti zadan poet byt a zkopruje je do zadan
; promnn. 
; Parametry:
;	Dest	- cl, kam se maj naten data zkoprovat (nap. clov promnn)
;	Offs	- Ofset v rmci zlohovan pamti, odkud se m st
;	Size	- Poet byt, kter se m st
DEF_T_MACRO PLC_MEM_READ Dest, Offs, Size
T_LOCAL	LBL_LOOP
		MOV		BX, Offs
		XOR		SI, SI
LBL_LOOP:
		MOV		AL, BYTE PTR PLC_MEM_BACKUP[BX][SI]
		MOV		BYTE PTR Dest[SI],AL				
		INC		SI
		CMP		SI, Size
		JB		LBL_LOOP
END_T_MACRO t_normal, c_0, d_on



; Makro PLC_MEM_WRITE slou k usnadnn zpisu do zlohovan pamti PLC (PLC_MEM_BACKUP).
; Zape do zadanho msta zlohovan pamti zadan poet byt ze zadan promnn. 
; Parametry:
;	Src		- zdroj, odkud se maj nast data ke zkoprovn do zlohovan pamti (nap. zdrojov promnn)
;	Offs	- Ofset v rmci zlohovan pamti, kam se m zapisovat
;	Size	- Poet byt, kter se m zapsat
DEF_T_MACRO PLC_MEM_WRITE Src, Offs, Size
T_LOCAL	LBL_LOOP
		MOV		BX, Offs
		XOR		SI, SI
LBL_LOOP:
		MOV		AL, BYTE PTR Src[SI]				
		MOV		BYTE PTR PLC_MEM_BACKUP[BX][SI],AL
		INC		SI
		CMP		SI, Size
		JB		LBL_LOOP
END_T_MACRO t_normal, c_0, d_on

; Makro ANALOG_FILTER pedstavuje digitln IIR filtr 1. du s monost
; nastaven mtka pro vstupn veliinu. Rovnice filtru:
;
;		y(n) = ( S * x(n) - y(n-1) ) * F + y(n-1) 
;
; Vstup: 	DR_REAL				x(n)	REAL
;       	rScale 				S 		REAL
;       	rFilterCoef 	 	F 		REAL
;			rMem				M   	REAL
;
; Vstup:	DR_REAL				y(n)	REAL
DEF_T_MACRO ANALOG_FILTER  rScale, rFilterCoef, rMem
			pushf
			cli
			finit
			fld		qword ptr DR_REAL
			fmul	qword ptr rScale		;meritko
			fsub	qword ptr rMem
			fmul	qword ptr rFilterCoef	;floating point
			fadd	qword ptr rMem
			fst		qword ptr rMem
			fstp	qword ptr DR_REAL
			wait
			popf
END_T_MACRO T_NORMAL, C_0, D_ON

; Makro SDO slou k odesln SDO paketu. Parametry:
;   Buf:		Buffer, ve kterm se odeslan paket sestav 
;   ID:			COB ID
;	Len:		Dlka dat
;   DataN:		Jednotliv byty dat
DEF_T_MACRO SDO Buf, ID, Len, Data0, Data1, Data2, Data3, Data4, Data5, Data6, Data7
      ; Sestavit paket
   LOD      TMAC.ID
   STO      WORD.Buf.CAN_ID
   LOD      CNST.0
   STO      BYTE.Buf.CAN_RTR
   LOD      TMAC.Len
   STO      BYTE.Buf.CAN_LEN
   LOD      TMAC.Data0
   STO      BYTE.Buf.CAN_DATA
   LOD      TMAC.Data1
   STO      BYTE.Buf.CAN_DATA_1
   LOD      TMAC.Data2
   STO      BYTE.Buf.CAN_DATA_2
   LOD      TMAC.Data3
   STO      BYTE.Buf.CAN_DATA_3
   LOD      TMAC.Data4
   STO      BYTE.Buf.CAN_DATA_4
   LOD      TMAC.Data5
   STO      BYTE.Buf.CAN_DATA_5
   LOD      TMAC.Data6
   STO      BYTE.Buf.CAN_DATA_6
   LOD      TMAC.Data7
   STO      BYTE.Buf.CAN_DATA_7
      ; Zaadit paket do fronty k odesln
   CAN_SEND 2, TMAC.Buf
END_T_MACRO T_NORMAL, C_0, D_ON

; Makro PLCOVR1_RDCLR umouje v jednom kroku nast a vynulovat hodnotu PLC overridu 1. Pro sprvnou 
; funkci mus bt tento override nastave tak, e m 201 hodnot, piem hodnota 0 je -0.1 (-100 promile) 
; a hodnota 200 je 0.1 (100 promile). Hodnota se nate do datovho registru.
DEF_T_MACRO	PLCOVR1_RDCLR
T_LOCAL LBL_PLCOVR1_RDCLR_KO, LBL_PLCOVR1_RDCLR_END
		; Zkusit si pivlastnit mutex pro zen pstupu ke struktue overridu
	MOV		AL, 0FFh
	XCHG	AL, PR1W.PRxW_BUSY
	OR		AL, AL
	JNZ		LBL_PLCOVR1_RDCLR_KO				; Pokud je mutex zamknut, je poteba ekat - skoit na LBL_PLCOVR1_RDCLR_KO
		; Vynulovat hodnotu overridu (nastavenm indexu odpovdajcmu nulov hodnot)
	LOD		CNST.100
	STO		BYTE.PR1W.PRxW_IDX
		; Nast hodnotu do DR
	LOD		WORD.PR1W.PRxW
		; Uvolnit mutex
	MOV		PR1W.PRxW_BUSY, 0
		; Dokoneno - do RLO nastavit 1 a skoit na konec
	MOV		AH, 0FFh
	JUM		LBL_PLCOVR1_RDCLR_END

LBL_PLCOVR1_RDCLR_KO:
		; Nepodailo se zskat vhradn pstup - do RLO nastavit 0
	MOV		AH, 0
		
LBL_PLCOVR1_RDCLR_END:
END_T_MACRO T_NORMAL, C_0, D_ON


